@@ -8,12 +8,12 @@
 #include <jni.h>
 #include <langinfo.h>
 #include <locale.h>
-#include <string.h>
 #include <stdio.h>
+#include <string.h>
 
-#ifdef USE_PTHREADS
-#include <pthread.h>
-#endif
+#ifdef HAVE_STDATOMIC_H
+#include <stdatomic.h>
+#endif /* HAVE_STDATOMIC_H */
 
 #include <link-grammar/api-structures.h>
 #include "link-grammar/corpus/corpus.h"
@@ -28,50 +28,22 @@
 /* Default to the English language. */
 static const char* in_language = "en";
 
+/* Dicationary can be and should be shared by all. */
+static Dictionary dict = NULL;
+#ifdef HAVE_STDATOMIC_H
+static atomic_flag dict_is_init = ATOMIC_FLAG_INIT;
+#else /* HAVE_STDATOMIC_H */
+static bool dict_is_init = false;
+#endif /* HAVE_STDATOMIC_H */
+
 typedef struct
 {
-	Dictionary    dict;
 	Parse_Options opts, panic_parse_opts;
 	Sentence      sent;
 	Linkage       linkage;
 	int           num_linkages, cur_linkage;
 } per_thread_data;
 
-/* XXX FIXME
- * The per_thread_data struct should ideally be somehow
- * fetched from JNIEnv, or as an opaque pointer in the class.
- * Not clear how to do this .. perhaps use NewDirectByteBuffer()
- * and the java.nio.ByteBuffer class ?
- */
-#ifdef USE_PTHREADS
-static pthread_key_t java_key;
-static pthread_once_t java_key_once = PTHREAD_ONCE_INIT;
-
-static void java_key_alloc(void)
-{
-	pthread_key_create(&java_key, free);
-}
-#else
-static per_thread_data * global_ptd = NULL;
-#endif
-
-
-static per_thread_data * get_ptd(JNIEnv *env, jclass cls)
-{
-#ifdef USE_PTHREADS
-	per_thread_data *ptd = pthread_getspecific(java_key);
-#else
-	per_thread_data *ptd = global_ptd;
-#endif
-	if (!ptd) Java_org_linkgrammar_LinkGrammar_init(env, cls);
-#ifdef USE_PTHREADS
-	ptd = pthread_getspecific(java_key);
-#else
-	ptd = global_ptd;
-#endif
-	return ptd;
-}
-
 static void setup_panic_parse_options(Parse_Options opts)
 {
 	parse_options_set_repeatable_rand(opts, false);
@@ -85,6 +57,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_linkage_limit(opts, 100);
 	parse_options_set_verbosity(opts, 0);
 	parse_options_set_spell_guess(opts, false);
+	parse_options_set_display_morphology(opts, true);
 }
 
 static inline void test(void)
@@ -112,10 +85,26 @@ static void throwException(JNIEnv *env, const char* message)
 		(*env)->FatalError(env, "Fatal: link-grammar JNI: Cannot throw");
 }
 
-static per_thread_data * init(JNIEnv *env, jclass cls)
+// Note that we do NOT offer any kind of protection from having
+// another thread access the dictionary while this thread is still
+// setting it up. It is up to the user to make sure that this thread
+// (more generally, that all threads that call init) return before
+// any of the threads call the parse functions.
+//
+// The only purpose of using the atomic_flag is to prevent two
+// different threads from accidentally trying to initialize the
+// dict at exactly the same time. (Which does seem like a reasonable
+// guarantee to offer).
+static void global_init(JNIEnv *env)
 {
+#ifdef HAVE_STDATOMIC_H
+	if (atomic_flag_test_and_set(&dict_is_init)) return;
+#else /* HAVE_STDATOMIC_H */
+	if (dict_is_init) return;
+	dict_is_init = true;
+#endif /* HAVE_STDATOMIC_H */
+
 	const char *codeset, *dict_version;
-	per_thread_data *ptd;
 
 	/* Get the locale from the environment...
 	 * perhaps we should someday get it from the dictionary ??
@@ -133,7 +122,21 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 		setlocale(LC_CTYPE, "en_US.UTF-8");
 	}
 
-	ptd = (per_thread_data *) malloc(sizeof(per_thread_data));
+	dict = dictionary_create_lang(in_language);
+	if (!dict) throwException(env, "Error: unable to open dictionary");
+	else test();
+
+	dict_version = linkgrammar_get_dict_version(dict);
+	prt_error("Info: JNI: dictionary language '%s' version %s\n",
+		in_language, dict_version);
+}
+
+static per_thread_data * init(JNIEnv *env)
+{
+	global_init(env);
+
+	per_thread_data *ptd = (per_thread_data *)
+		malloc(sizeof(per_thread_data));
 	memset(ptd, 0, sizeof(per_thread_data));
 
 	ptd->opts = parse_options_create();
@@ -155,17 +158,11 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 	parse_options_set_short_length(ptd->opts, 16);
 	parse_options_set_verbosity(ptd->opts, 0);
 	parse_options_set_spell_guess(ptd->opts, false);
+	parse_options_set_display_morphology(ptd->opts, true);
 
 	ptd->panic_parse_opts = parse_options_create();
 	setup_panic_parse_options(ptd->panic_parse_opts);
 
-	ptd->dict = dictionary_create_lang(in_language);
-	if (!ptd->dict) throwException(env, "Error: unable to open dictionary");
-	else test();
-
-	dict_version = linkgrammar_get_dict_version(ptd->dict);
-	prt_error("Info: JNI: dictionary version %s\n", dict_version);
-
 	return ptd;
 }
 
@@ -179,27 +176,26 @@ static void finish(per_thread_data *ptd)
 		linkage_delete(ptd->linkage);
 	ptd->linkage = NULL;
 
-	dictionary_delete(ptd->dict);
-	ptd->dict = NULL;
-
 	parse_options_delete(ptd->opts);
 	ptd->opts = NULL;
 
 	parse_options_delete(ptd->panic_parse_opts);
 	ptd->panic_parse_opts = NULL;
 
-#ifdef USE_PTHREADS
-	pthread_setspecific(java_key, NULL);
-#else
-	global_ptd = NULL;
-#endif
 	free(ptd);
 }
 
+static TLS per_thread_data * local_ptd = NULL;
+static per_thread_data * get_ptd(JNIEnv *env, jclass cls)
+{
+	if (!local_ptd) local_ptd = init(env);
+	return local_ptd;
+}
+
 /* ================================================================= */
 /* Misc utilities */
 
-static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
+static void jParse(JNIEnv *env, per_thread_data *ptd, const char* inputString)
 {
 	Parse_Options opts = ptd->opts;
 	int jverbosity = parse_options_get_verbosity(opts);
@@ -212,9 +208,9 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 	if (ptd->sent)
 		sentence_delete(ptd->sent);
 
-	if (ptd->dict == NULL) throwException(env, "jParse: dictionary not open\n");
+	if (dict == NULL) throwException(env, "jParse: dictionary not open\n");
 	if (inputString == NULL) throwException(env, "jParse: no input sentence!\n");
-	ptd->sent = sentence_create(inputString, ptd->dict);
+	ptd->sent = sentence_create(inputString, dict);
 	ptd->num_linkages = 0;
 
 	if (ptd->sent == NULL)
@@ -270,6 +266,11 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 	}
 }
 
+void unit_test_jparse(JNIEnv *env, const char* inputString)
+{
+	jParse(env, get_ptd(env, 0), inputString);
+}
+
 static void makeLinkage(per_thread_data *ptd)
 {
 	if (ptd->cur_linkage < ptd->num_linkages)
@@ -300,8 +301,8 @@ Java_org_linkgrammar_LinkGrammar_getVersion(JNIEnv *env, jclass cls)
 JNIEXPORT jstring JNICALL
 Java_org_linkgrammar_LinkGrammar_getDictVersion(JNIEnv *env, jclass cls)
 {
-	per_thread_data *ptd = get_ptd(env, cls);
-	const char *s = linkgrammar_get_dict_version(ptd->dict);
+	init(env);
+	const char *s = linkgrammar_get_dict_version(dict);
 	jstring j = (*env)->NewStringUTF(env, s);
 	return j;
 }
@@ -374,17 +375,9 @@ Java_org_linkgrammar_LinkGrammar_getMaxLinkages(JNIEnv *env, jclass cls)
 JNIEXPORT void JNICALL
 Java_org_linkgrammar_LinkGrammar_init(JNIEnv *env, jclass cls)
 {
-#ifdef USE_PTHREADS
-	per_thread_data *ptd;
-	pthread_once(&java_key_once, java_key_alloc);
-	ptd = pthread_getspecific(java_key);
-	if (ptd) return;
-	ptd = init(env, cls);
-	pthread_setspecific(java_key, ptd);
-#else
-	if (global_ptd) return;
-	global_ptd = init(env, cls);
-#endif
+	// Force global init, always.
+	global_init(env);
+	get_ptd(env, cls);
 }
 
 /*
@@ -413,8 +406,32 @@ Java_org_linkgrammar_LinkGrammar_parse(JNIEnv *env, jclass cls, jstring str)
 JNIEXPORT void JNICALL
 Java_org_linkgrammar_LinkGrammar_close(JNIEnv *env, jclass cls)
 {
-	per_thread_data *ptd = get_ptd(env, cls);
-	finish(ptd);
+	if (!local_ptd) return;
+	finish(local_ptd);
+	local_ptd = NULL;
+}
+
+/*
+ * Class:      LinkGrammar
+ * Method:     finalize
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL
+Java_org_linkgrammar_LinkGrammar_doFinalize(JNIEnv *env, jclass cls)
+{
+	if (local_ptd) finish(local_ptd);
+	local_ptd = NULL;
+
+	// Note that we do NOT offer any protection from having another
+	// thread access the dictionary, while this thread is finalizing.
+	// It is up to the user to avoid this kind of mistake!
+	if (dict) dictionary_delete(dict);
+	dict = NULL;
+#ifdef HAVE_STDATOMIC_H
+	atomic_flag_clear(&dict_is_init);
+#else /* HAVE_STDATOMIC_H */
+	dict_is_init = false;
+#endif /* HAVE_STDATOMIC_H */
 }
 
 /*
@@ -435,7 +452,7 @@ Java_org_linkgrammar_LinkGrammar_getLinkageWord(JNIEnv *env, jclass cls, jint i)
 	per_thread_data *ptd = get_ptd(env, cls);
 
 	/* Does not need to be freed, points into data structures */
-	/* Returns the inflected word. */
+	/* Returns the subscripted word. */
 	const char * w = linkage_get_word(ptd->linkage, i);
 	jstring j = (*env)->NewStringUTF(env, w);
 	return j;
