@@ -68,6 +68,8 @@ JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_init
 JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_parse
 	(JNIEnv *, jclass, jstring);
 
+void unit_test_jparse(JNIEnv *, const char*);
+
 /*
  * Class:     LinkGrammar
  * Method:    close
@@ -76,6 +78,14 @@ JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_parse
 JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_close
 	(JNIEnv *, jclass);
 
+/*
+ * Class:     LinkGrammar
+ * Method:    finalize
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_doFinalize
+	(JNIEnv *, jclass);
+
 /*
  * Class:     LinkGrammar
  * Method:    numWords
