@@ -10,8 +10,11 @@
 /*                                                                       */
 /*************************************************************************/
 
+#ifndef _ANALYZE_LINKAGE_H
+#define _ANALYZE_LINKAGE_H
+
 #include "api-types.h"
 #include "link-includes.h"
 
 void compute_link_names(Linkage, String_set *);
-
+#endif /* _ANALYZE_LINKAGE_H */
