@@ -154,8 +154,6 @@ static int revcmplen(const void *a, const void *b)
  * and prefixes - every time we see a new suffix/prefix (the previous one is
  * remembered by w_last), we save it in the corresponding affix-class list.
  * The saved affixes don't include the infix mark.
- *
- * The empty word is not an affix so it is ignored.
  */
 static void get_dict_affixes(Dictionary dict, Dict_node * dn,
                              char infix_mark, char * w_last)
@@ -183,8 +181,7 @@ static void get_dict_affixes(Dictionary dict, Dict_node * dn,
 		strncpy(w_last, w, w_len);
 		w_last[w_len] = '\0';
 
-		if ((infix_mark == w_last[0]) &&
-			 (0 != strcmp(w_last, EMPTY_WORD_MARK)))
+		if (infix_mark == w_last[0])
 		{
 			affix_list_add(afdict, &afdict->afdict_class[AFDICT_SUF], w_last+1);
 		}
@@ -589,8 +586,6 @@ dictionary_six_str(const char * lang,
 	dict->left_wall_defined  = boolean_dictionary_lookup(dict, LEFT_WALL_WORD);
 	dict->right_wall_defined = boolean_dictionary_lookup(dict, RIGHT_WALL_WORD);
 
-	dict->empty_word_defined = boolean_dictionary_lookup(dict, EMPTY_WORD_MARK);
-
 	dict->base_knowledge  = pp_knowledge_open(pp_name);
 	if (NULL == dict->base_knowledge) goto failure;
 	dict->hpsg_knowledge  = pp_knowledge_open(cons_name);
@@ -600,7 +595,7 @@ dictionary_six_str(const char * lang,
 	dict->use_unknown_word = true;
 
 	dict->shuffle_linkages = false;
-	if (0 == strcmp(dict->lang, "any") || NULL != dict->anysplit)
+	if (0 == strcmp(dict->lang, "any") || NULL != dict->affix_table->anysplit)
 		dict->shuffle_linkages = true;
 
 	dict_node = dictionary_lookup_list(dict, UNLIMITED_CONNECTORS_WORD);
