@@ -33,7 +33,6 @@ void free_disjuncts(Disjunct *c)
 		c1 = c->next;
 		free_connectors(c->left);
 		free_connectors(c->right);
-		free(c->word);
 		xfree((char *)c, sizeof(Disjunct));
 	}
 }
@@ -96,18 +95,6 @@ static inline unsigned int old_hash_disjunct(disjunct_dup_table *dt, Disjunct *
 	return (i & (dt->dup_table_size-1));
 }
 
-/* FIXME? Can the same word get appended again? If so - prevent it. */
-static void disjuct_gwordlist_append(Disjunct * dx, const Gword **word)
-{
-	size_t to_word_arr_len = gwordlist_len(dx->word);
-	size_t from_word_arr_len = gwordlist_len(word);
-
-	dx->word = realloc(dx->word,
-	   sizeof(*(dx->word)) * (to_word_arr_len + from_word_arr_len + 1));
-	memcpy(&dx->word[to_word_arr_len], word,
-	       sizeof(*dx->word) * (from_word_arr_len + 1));
-}
-
 /**
  * The connectors must be exactly equal.  A similar function
  * is connectors_equal_AND(), but that ignores priorities,
@@ -190,9 +177,7 @@ Disjunct *disjuncts_dup(Disjunct *origd)
 		newd->cost = t->cost;
 		newd->left = connectors_dup(t->left);
 		newd->right = connectors_dup(t->right);
-		newd->word = NULL;
-		disjuct_gwordlist_append(newd, t->word);
-
+		newd->originating_gword = t->originating_gword;
 		prevd->next = newd;
 		prevd = newd;
 	}
@@ -221,6 +206,84 @@ static void disjunct_dup_table_delete(disjunct_dup_table *dt)
 	xfree(dt, sizeof(disjunct_dup_table));
 }
 
+#ifdef DEBUG
+GNUC_UNUSED static int gword_set_len(const gword_set *gl)
+{
+	int len = 0;
+	for (; NULL != gl; gl = gl->next) len++;
+	return len;
+}
+#endif
+
+/**
+ * Return a new gword_set element, initialized from the given element.
+ * @old_e Existing element.
+ */
+static gword_set *gword_set_element_new(gword_set *old_e)
+{
+	gword_set *new_e = malloc(sizeof(gword_set));
+	*new_e = (gword_set){0};
+
+	new_e->o_gword = old_e->o_gword;
+	gword_set *chain_next = old_e->chain_next;
+	old_e->chain_next = new_e;
+	new_e->chain_next = chain_next;
+
+	return new_e;
+}
+
+/**
+ * Add an element to existing gword_set. Uniqueness is assumed.
+ * @return A new set with the element.
+ */
+static gword_set *gword_set_add(gword_set *gset, gword_set *ge)
+{
+	gword_set *n = gword_set_element_new(ge);
+	n->next = gset;
+	gset = n;
+
+	return gset;
+}
+
+/**
+ * Combine the given gword sets.
+ * The gword sets are not modified.
+ * This function is used for adding the gword pointers of an eliminated
+ * disjunct to the ones of the kept disjuncts, with no duplicates.
+ *
+ * @kept gword_set of the kept disjunct.
+ * @eliminated gword_set of the eliminated disjunct.
+ * @return Use copy-on-write semantics - the gword_set of the kept disjunct
+ * just gets returned if there is nothing to add to it. Else - a new gword
+ * set is returned.
+ */
+static gword_set *gword_set_union(gword_set *kept, gword_set *eliminated)
+{
+	/* Preserve the gword pointers of the eliminated disjunct if different. */
+	gword_set *preserved_set = NULL;
+	for (gword_set *e = eliminated; NULL != e; e = e->next)
+	{
+		gword_set *k;
+
+		/* Ensure uniqueness. */
+		for (k = kept; NULL != k; k = k->next)
+			if (e->o_gword == k->o_gword) break;
+		if (NULL != k) continue;
+
+		preserved_set = gword_set_add(preserved_set, e);
+	}
+
+	if (preserved_set)
+	{
+		/* Preserve the originating gword pointers of the remaining disjunct. */
+		for (gword_set *k = kept; NULL != k; k = k->next)
+			preserved_set = gword_set_add(preserved_set, k);
+		kept = preserved_set;
+	}
+
+	return kept;
+}
+
 /**
  * Takes the list of disjuncts pointed to by d, eliminates all
  * duplicates, and returns a pointer to a new list.
@@ -253,7 +316,10 @@ Disjunct * eliminate_duplicate_disjuncts(Disjunct * d)
 		{
 			d->next = NULL;  /* to prevent it from freeing the whole list */
 			if (d->cost < dx->cost) dx->cost = d->cost;
-			disjuct_gwordlist_append(dx, d->word);
+
+			dx->originating_gword =
+				gword_set_union(dx->originating_gword, d->originating_gword);
+
 			free_disjuncts(d);
 			count++;
 		}
@@ -324,40 +390,7 @@ char * print_one_disjunct(Disjunct *dj)
 void word_record_in_disjunct(const Gword * gw, Disjunct * d)
 {
 	for (;d != NULL; d=d->next) {
-		d->word = malloc(sizeof(*d->word)*2);
-		d->word[0] = gw;
-		d->word[1] = NULL;
+		d->originating_gword = (gword_set *)&gw->gword_set_head;
 	}
 }
-
-/* Debug printout of a disjunct word list */
-void disjunct_word_print(Disjunct * d)
-{
-	const Gword *const *wl = d->word;
-	const char *c;
-
-	printf("Disjunct word: ");
-	for (c = d->string; '\0' != *c; c++)
-		printf("%c", *c == SUBSCRIPT_MARK ? '.' : *c);
-	printf("; Wordgraph: ");
-
-	if (NULL == wl)
-	{
-		printf("BUG: NULL list!\n");
-		return;
-	}
-	if (NULL == *wl)
-	{
-		printf("BUG: None\n");
-		return;
-	}
-
-	for (; *wl; wl++)
-	{
-		printf("word '%s' unsplit '%s'%s", NULL == *(wl+1) ? "" : ",",
-		       (*wl)->subword, (*wl)->unsplit_word->subword);
-	}
-	printf("\n");
-}
-
 /* ========================= END OF FILE ============================== */
