@@ -12,4 +12,4 @@
 #include "api-types.h"
 #include "link-includes.h"
 
-void lg_compute_disjunct_strings(Sentence, Linkage_info *);
+void lg_compute_disjunct_strings(Linkage);
