@@ -10,6 +10,9 @@
 /*                                                                       */
 /*************************************************************************/
 
+#ifndef _EXTRACT_LINKS_H
+#define _EXTRACT_LINKS_H
+
 #include "api-structures.h"
 #include "link-includes.h"
 
@@ -17,3 +20,4 @@ Parse_info parse_info_new(int nwords);
 void free_parse_info(Parse_info);
 bool build_parse_set(Sentence, fast_matcher_t*, count_context_t*, unsigned int null_count, Parse_Options);
 void extract_links(Linkage, Parse_info);
+#endif /* _EXTRACT_LINKS_H */
