@@ -4,12 +4,15 @@
 /* All rights reserved                                                          */
 /*                                                                              */
 /* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */ 
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
+/* license set forth in the LICENSE file included with this software.           */ 
 /* This license allows free redistribution and use in source and binary         */
 /* forms, with or without modification, subject to certain conditions.          */
 /*                                                                              */
 /********************************************************************************/
+
+#include "dict-structures.h"
+#include "link-includes.h"
+
 void insert_idiom(Dictionary dict, Dict_node *);
-Boolean  contains_underbar(const char *);
-Boolean  is_idiom_word(const char *);
+bool  contains_underbar(const char *);
+bool  is_idiom_word(const char *);
