@@ -129,23 +129,10 @@ static Gword *wordgraph_null_join(Sentence sent, Gword **start, Gword **end)
  * for working with linkages.
  */
 
-/*
- * The "empty word" is a concept used in order to make the current parser able
- * to parse "alternatives" within a sentence. The "empty word" can link to any
- * word, hence it is issued when a word is optional, a thing that happens when
- * there are alternatives with different number of tokens in each of them.
- *
- * However, the empty words are not needed after the linkage step, and so,
- * below, we remove them from the linkage, as well as the links that
- * connect to them.
- *
- * When the word-graph is converted ("flattened") to the 2D array used by
- * the parser, empty words are issued whenever needed.  This essentially
- * means that the empty word (EMPTY-WORD.zzz) must be defined in the
- * dictionary of every language.
- */
 #define SUBSCRIPT_SEP SUBSCRIPT_DOT /* multiple-subscript separator */
 
+#define PREFIX_SUPPRESS ("PL") /* prefix links start with this */
+#define PREFIX_SUPPRESS_L 2    /* length of above */
 #define SUFFIX_SUPPRESS ("LL") /* suffix links start with this */
 #define SUFFIX_SUPPRESS_L 2    /* length of above */
 
@@ -156,37 +143,46 @@ static Gword *wordgraph_null_join(Sentence sent, Gword **start, Gword **end)
 /* FIXME: Define an affix class MORPHOLOGY_LINKS. */
 static inline bool is_morphology_link(const char *link_name)
 {
-	return (NULL != link_name) &&
-	        (0 == strncmp(link_name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L));
+	if (NULL == link_name) return false;
+	return (0 == strncmp(link_name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L)) ||
+	       (0 == strncmp(link_name, PREFIX_SUPPRESS, PREFIX_SUPPRESS_L));
 }
 
 /*
  * Remap the link array according to discarded links and words.
- * If pp_info exists, remap it too.
  *
  * The remap[] elements indicate the new WordIdx of the word.
  * A value which is -1 indicates a discarded word.
  * A NULL link_name indicates a discarded link.
  */
-void remap_linkages(Linkage lkg, const int *remap)
+static void remap_linkages(Linkage lkg, const int *remap)
 {
-	LinkIdx i, j, k;
+	LinkIdx i, j;
 
 	for (i = 0, j = 0; i < lkg->num_links; i++)
 	{
-		const Link *old_lnk = &lkg->link_array[i];
+		Link *old_lnk = &lkg->link_array[i];
 
 		if (NULL != old_lnk->link_name &&  /* discarded link */
 		   (-1 != remap[old_lnk->rw]) && (-1 != remap[old_lnk->lw]))
 		{
 			Link *new_lnk = &lkg->link_array[j];
+			Connector *ctmp;
 
 			/* Copy the entire link contents, thunking the word numbers.
 			 * Note that j is always <= i so this is always safe. */
+
 			new_lnk->lw = remap[old_lnk->lw];
 			new_lnk->rw = remap[old_lnk->rw];
+
+			ctmp = new_lnk->lc;
 			new_lnk->lc = old_lnk->lc;
+			old_lnk->lc = ctmp;
+
+			ctmp = new_lnk->rc;
 			new_lnk->rc = old_lnk->rc;
+			old_lnk->rc = ctmp;
+
 			new_lnk->link_name = old_lnk->link_name;
 
 			/* Remap the pp_info, too. */
@@ -203,17 +199,12 @@ void remap_linkages(Linkage lkg, const int *remap)
 		}
 	}
 
-	/* Zero out the rest of the domain name array
-	 * (its already been copied to a new location, above). */
-	if (lkg->pp_info)
-		for (k=j; k<i; k++) lkg->pp_info[k].num_domains = 0;
-
 	lkg->num_links = j;
 	/* Unused memory not freed - all of it will be freed in free_linkages(). */
 }
 
 /**
- * Remove the empty words from a linkage.
+ * Remove unlinked optional words from a linkage.
  * XXX Should we remove here also the dict-cap tokens? In any case, for now they
  * are left for debug.
  */
@@ -232,27 +223,31 @@ void remove_empty_words(Linkage lkg)
 
 	for (i = 0, j = 0; i < lkg->num_words; i++)
 	{
-		if ((NULL != cdj[i]) && (MT_EMPTY == cdj[i]->word[0]->morpheme_type))
+		if ((NULL == cdj[i]) && lkg->sent->word[i].optional)
 		{
 			remap[i] = -1;
 		}
 		else
 		{
+			Disjunct *cdtmp = cdj[j];
 			cdj[j] = cdj[i];
+			cdj[i] = cdtmp; /* The SAT parser frees chosen_disjuncts elements. */
 			remap[i] = j;
 			j++;
 		}
 	}
-	lkg->num_words = j;
-	/* Unused memory not freed - all of it will be freed in free_linkages(). */
+	if (lkg->num_words != j)
+	{
+		/* Unused memory not freed - all of it will be freed in free_linkages(). */
+		lkg->num_words = j;
+		remap_linkages(lkg, remap); /* Update lkg->link_array and lkg->num_links. */
+	}
 
 	if (verbosity_level(+D_REE))
 	{
 		err_msg(lg_Debug, "chosen_disjuncts after:\n\\");
 		print_chosen_disjuncts_words(lkg);
 	}
-
-	remap_linkages(lkg, remap); /* Update lkg->link_array and lkg->num_links. */
 }
 #undef D_REE
 
@@ -424,8 +419,6 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 			/* TODO: Suppress "virtual-morphemes", currently the dictcap ones. */
 			char *sm;
 
-			assert(MT_EMPTY != cdj->word[0]->morpheme_type);/* already discarded */
-
 			t = cdj->string;
 			/* Print the subscript, as in "dog.n" as opposed to "dog". */
 
@@ -566,8 +559,9 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 			if (!join_alt) gwordlist_append(&n_lwg_path, *wgp);
 
 			/*
-			 * Add guess marks in [] if needed, at the end of the base word.
-			 * Convert the badly-printing ^C into a period.
+			 * Add guess marks in [] square brackets, if needed, at the
+			 * end of the base word. Convert the badly-printing
+			 * SUBSCRIPT_MARK (hex 03 or ^C) into a period.
 			 */
 			if (t)
 			{
@@ -685,7 +679,9 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 		if (chosen_words[i] &&
 		    (chosen_words[i][0] || (!HIDE_MORPHO || show_word[i])))
 		{
+			const char *cwtmp = linkage->word[j];
 			linkage->word[j] = chosen_words[i];
+			chosen_words[i] = cwtmp;
 			remap[i] = j;
 			j++;
 		}
@@ -727,7 +723,6 @@ Linkage linkage_create(LinkageIdx k, Sentence sent, Parse_Options opts)
 	/* Perform remaining initialization we haven't done yet...*/
 	compute_chosen_words(sent, linkage, opts);
 
-	linkage->sent = sent;
 	linkage->is_sent_long = (linkage->num_words >= opts->twopass_length);
 
 	return linkage;
