@@ -28,6 +28,9 @@
 #include "api-types.h"
 #include "structures.h"
 
+void pp_new_domain_array(PP_data *);
+void pp_free_domain_array(PP_data *);
+
 Postprocessor * post_process_new(pp_knowledge *);
 void post_process_free(Postprocessor *);
 
@@ -38,4 +41,10 @@ bool     post_process_match(const char *, const char *);  /* utility function */
 
 bool sane_linkage_morphism(Sentence, Linkage, Parse_Options);
 
+void linkage_free_pp_info(Linkage);
+
+void build_type_array(Postprocessor*);
+void linkage_set_domain_names(Postprocessor*, Linkage);
+void exfree_domain_names(PP_info *);
+
 #endif
