@@ -20,6 +20,7 @@
 #endif
 
 #include <stdlib.h>
+#include <stdarg.h>
 
 typedef struct String_s String;
 
@@ -28,6 +29,8 @@ void string_delete(String *);
 const char * string_value(String *);
 char * string_copy(String *);
 void append_string(String * string, const char *fmt, ...) GNUC_PRINTF(2,3);
+void vappend_string(String * string, const char *fmt, va_list args)
+	GNUC_PRINTF(2,0);
 size_t append_utf8_char(String * string, const char * mbs);
 
 #endif
