@@ -428,6 +428,9 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	char xpicture[MAX_HEIGHT][MAX_LINE];
 	size_t start[MAX_HEIGHT];
 
+	// Avoid pathological case and the resulting crash.
+	if (0 == linkage->num_words) return strdup("");
+
 	string = string_new();
 
 	/* Do we want to print the left wall? */
@@ -603,7 +606,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 		}
 		xpicture[1][k] = '\0';
 		for (row=0; row < top_row+1; row++) {
-			strcpy(xpicture[row+2],picture[row]);
+			strcpy(xpicture[row+2], picture[row]);
 		}
 		top_row += 2;
 	} else {
@@ -1272,10 +1275,10 @@ void print_sentence_word_alternatives(Sentence sent, bool debugprint,
 		 *   ה=  כלב  לב  ב=
 		 *   ה=  כ=  ל=
 		 *   ה=  כ=
-		 * For "'50s," (∅ means empty word):
+		 * For "'50s,"
 		 *   ' s s ,
-		 *   '50 50 , ∅
-		 *   '50s ∅
+		 *   '50 50 ,
+		 *   '50s
 		 * Clearly, this is not informative any more. Instead, one line with a
 		 * list of tokens (without repetitions) is printed
 		 * ה= כלב לב ב= כ= ל=
@@ -1329,13 +1332,8 @@ void print_sentence_word_alternatives(Sentence sent, bool debugprint,
 				const char *st = NULL;
 				char *wprint = NULL;
 
-				/* In the Wordgraph version alternative slots are not balanced
-				 * with empty words. To see these places for debug, later
-				 * here "[missing]" is printed if wt is "". */
-				if (ai >= nalts)
-					wt = ""; /* missing */
-				else
-					wt = sent->word[wi].alternatives[ai];
+				if (ai >= nalts) continue;
+				wt = sent->word[wi].alternatives[ai];
 
 				/* Don't display information again for the same word */
 				if ((NULL != tokenpos) && (0 == strcmp(tokenpos->token, wt)))
@@ -1359,9 +1357,6 @@ void print_sentence_word_alternatives(Sentence sent, bool debugprint,
 					}
 				}
 
-				if (0 == strcmp(wt, EMPTY_WORD_MARK))
-					wt = EMPTY_WORD_DISPLAY;
-
 				/* Restore SUBSCRIPT_DOT for printing */
 				st = strrchr(wt, SUBSCRIPT_MARK);
 				if (st)
@@ -1372,10 +1367,19 @@ void print_sentence_word_alternatives(Sentence sent, bool debugprint,
 					wt = wprint;
 				}
 
-				if (debugprint) lgdebug(0, " %s", '\0' == wt[0] ? "[missing]" : wt);
+				if (debugprint)
+				{
+					const char *opt_start = "", *opt_end = "";
+					if (sent->word[wi].optional)
+					{
+						opt_start = "{";
+						opt_end = "}";
+					}
+					lgdebug(0, " %s%s%s", opt_start, wt, opt_end);
+				}
 
 				/* Don't try to give info on the empty word. */
-				if (('\0' != wt[0]) && (0 != strcmp(wt, EMPTY_WORD_DISPLAY)))
+				if ('\0' != wt[0])
 				{
 					/* For now each word component is called "Token".
 					 * TODO: Its type can be decoded and a more precise
@@ -1410,7 +1414,7 @@ void print_with_subscript_dot(const char *s)
 	const char *mark = strchr(s, SUBSCRIPT_MARK);
 	size_t len = NULL != mark ? (size_t)(mark - s) : strlen(s);
 
-	printf("%.*s%s%s ", (int)len,
+	prt_error("%.*s%s%s ", (int)len,
 			  s, NULL != mark ? "." : "", NULL != mark ? mark+1 : "");
 }
 
@@ -1460,9 +1464,7 @@ void print_chosen_disjuncts_words(const Linkage lkg)
 		const char *djw; /* disjunct word - the chosen word */
 
 		if (NULL == cdj)
-			djw = "[]"; /* null word */
-		else if (MT_EMPTY == cdj->word[0]->morpheme_type)
-			djw = EMPTY_WORD_DISPLAY;
+			djw = lkg->sent->word[i].optional ? "{}" : "[]";
 		else if ('\0' == cdj->string[0])
 			djw = "\\0"; /* null string - something is wrong */
 		else
