@@ -9,8 +9,12 @@
 /*                                                                       */
 /*************************************************************************/
 
+#ifndef _REGEX_MORPH_H
+#define _REGEX_MORPH_H
+
 #include "api-structures.h"
 
 int compile_regexs(Regex_node *, Dictionary);
 const char *match_regex(const Regex_node *, const char *);
 void free_regexs(Regex_node *);
+#endif /* _REGEX_MORPH_H */
