@@ -27,7 +27,7 @@ using std::endl;
 
 
 #ifdef _VARS
-extern ostream& var_defs_stream;
+extern std::ostream& var_defs_stream;
 #endif
 
 static char* construct_link_label(const char* connector1, const char* connector2) {
@@ -91,6 +91,16 @@ public:
    * General purpose variables specified by their names
    */
 
+  bool var_exists(const char* name) {
+    try {
+      int num = _variable_trie.lookup(name);
+      return num != Trie<int>::NOT_FOUND;
+      } catch (const std::string& s) {
+          cout << s << endl;
+          exit(EXIT_FAILURE);
+        }
+  }
+
   // If guiding params are unknown, they are set do default
   int string(const char* name)
   {
@@ -173,6 +183,7 @@ public:
     return var;
   }
 
+#if 0
   // If guiding params are unknown, they are set do default
   int linked_min(int wi, int wj) {
     int var;
@@ -185,8 +196,7 @@ public:
     assert(var != -1, "Var == -1");
     return var;
   }
-
-
+#endif
 
   /*
    *                  link(wi, pi, wj, pj)
@@ -226,6 +236,7 @@ public:
     return var;
   }
 
+#if 0
   /*
    *             thin_link(wi, wj)
    * Variables that specify that two words are linked by a thin link
@@ -244,6 +255,7 @@ public:
     assert(var != -1, "Var == -1");
     return var;
   }
+#endif
 
   /*
    *                   link_cw(wi, wj, pj)
@@ -415,10 +427,12 @@ public:
     const char* connector;
   };
 
+#if 0
   // Returns additional info about the given link_top_cw variable
   const LinkTopCWVar* link_top_cw_variable(int var) const {
     return _link_top_cw_variables[var];
   }
+#endif
 
   /* Pass SAT search parameters to the MiniSAT solver */
   void setVariableParameters(Solver* solver) {
@@ -673,9 +687,11 @@ private:
     return get_3int_variable(i, j, pj, var, _link_cw_variable_map);
   }
 
+#if 0
   bool get_link_top_cw_variable(int i, int j, int pj, int& var) {
     return get_3int_variable(i, j, pj, var, _link_top_cw_variable_map);
   }
+#endif
 
 
 #ifdef _CONNECTIVITY_
