@@ -6,14 +6,11 @@
 #include <set>
 
 extern "C" {
-#include "count.h"
-#include "prune.h"
 #include "word-utils.h"
 };
 
 #include "variables.hpp"
 
-
 struct PositionConnector
 {
   PositionConnector(Exp* e, Connector* c, char d, int w, int p, 
@@ -109,9 +106,14 @@ private:
   }
 
 public:
-  WordTag(int word, Variables* variables, Sentence sent, Parse_Options opts)
+  WordTag(int word, const char* name, Variables* variables, Sentence sent, Parse_Options opts)
     : _word(word), _variables(variables), _sent(sent), _opts(opts) {
     _match_possible.resize(_sent->length);
+
+    // The SAT word variables are set to be equal to the word numbers.
+    Var var = _variables->string(name);
+    assert(word == var);
+
     verbosity = opts->verbosity;
     debug = opts->debug;
     test = opts->test;
