@@ -42,9 +42,6 @@
  */
 #define SUBSCRIPT_MARK '\3'
 #define SUBSCRIPT_DOT '.'
-#define EMPTY_WORD_DOT   "EMPTY-WORD.zzz"  /* Has SUBSCRIPT_DOT in it! */
-#define EMPTY_WORD_MARK  "EMPTY-WORD\3zzz" /* Has SUBSCRIPT_MARK in it! */
-#define EMPTY_WORD_DISPLAY "∅"   /* Empty word representation for debug */
 #define EMPTY_CONNECTOR "ZZZ"
 
 /* Dictionary capitalization handling */
@@ -86,6 +83,33 @@
 #define SHORT_LEN 6
 #define NO_WORD 255
 
+/* An ordered set of gword pointers, used to indicate the source gword
+ * (Wordgraph word) of disjuncts and connectors. Usually it contains only
+ * one element.  However, when a duplicate disjunct is eliminated (see
+ * eliminate_duplicate_disjuncts()) and it originated from a different
+ * gword (a relatively rare event) its gword is added to the gword_set of
+ * the remaining disjunct. A set of 3 elements is extremely rare. The
+ * original order is preserved, in a hope for better caching on
+ * alternatives match checks in fast-match.c.
+ *
+ * Memory management:
+ * A copy-on-write semantics is used when constructing a new gword_set.  It
+ * means that all the gword sets with one element are shared.  These gword
+ * sets are part of the Gword structure. Copied and added element are
+ * alloc'ed and chained. The result is that the chain_next of the gword
+ * sets that are part of each gword contains the list of alloc'ed elements -
+ * to be used in gword_set_delete() called *only* in sentence_delete().
+ * This ensures that the gword_set of connectors doesn't get stale when
+ * their disjuncts are deleted and later restored in one-parse when
+ * min_null_count=0 and max_null count>0 (see classic_parse()).
+ */
+typedef struct gword_set
+{
+	Gword *o_gword;
+	struct gword_set *next;
+	struct gword_set *chain_next;
+} gword_set;
+
 /* On a 64-bit machine, this struct should be exactly 4*8=32 bytes long.
  * Lets try to keep it that way.
  */
@@ -111,7 +135,11 @@ struct Connector_struct
 	const char * string; /* The connector name w/o the direction mark, e.g. AB */
 
 	/* Hash table next pointer, used only during pruning. */
-	Connector * tableNext;
+	union
+	{
+		Connector * tableNext;
+		const gword_set *originating_gword;
+	};
 };
 
 static inline void connector_set_string(Connector *c, const char *s)
@@ -138,7 +166,7 @@ struct Disjunct_struct
 #ifdef VERIFY_MATCH_LIST
 	int match_id;              /* verify the match list integrity */
 #endif
-	const Gword **word;        /* NULL terminated list of originating words */
+	gword_set *originating_gword; /* List of originating gwords */
 	const char * string;       /* subscripted dictionary word */
 };
 
@@ -176,6 +204,7 @@ struct Word_struct
 
 	X_node * x;          /* Sentence starts out with these, */
 	Disjunct * d;        /* eventually these get generated. */
+	bool optional;       /* Linkage is optional. */
 
 	const char **alternatives;
 };
@@ -187,7 +216,7 @@ typedef enum
 	MT_FEATURE,            /* Pseudo morpheme, currently capitalization marks */
 	MT_INFRASTRUCTURE,     /* Start and end Wordgraph pseudo-words */
 	MT_WALL,               /* The LEFT-WALL and RIGHT-WALL pseudo-words */
-	MT_EMPTY,              /* Empty word */
+	MT_EMPTY,              /* Empty word FIXME: Remove it. */
 	MT_UNKNOWN,            /* Unknown word (FIXME? Unused) */
 	/* Experimental for Semitic languages (yet unused) */
 	MT_TEMPLATE,
@@ -206,7 +235,7 @@ typedef enum
 
 /* Word status */
 /* - Tokenization */
-#define WS_UNKNOWN (1<<0) /* Unknown word (FIXME? Unused) */
+#define WS_UNKNOWN (1<<0) /* Unknown word */
 #define WS_REGEX   (1<<1) /* Matches a regex */
 #define WS_SPELL   (1<<2) /* Result of a spell guess */
 #define WS_RUNON   (1<<3) /* Separated from words run-on */
@@ -224,10 +253,7 @@ typedef enum
 
 #define WS_GUESS (WS_SPELL|WS_RUNON|WS_REGEX)
 
-/* XXX Only TS_DONE is now actually used.
- * FIXME: Change TS_DONE to WS_TDONE, or
- * change Tokenizing_step to "bool tokenizing_step". */
-
+/* XXX Only TS_ANYSPLIT and TS_DONE are actually used. */
 typedef enum
 {
 	TS_INITIAL,
@@ -236,6 +262,7 @@ typedef enum
 	TS_REGEX,
 	TS_RUNON,
 	TS_SPELL,
+	TS_ANYSPLIT,             /* After anysplit */
 	TS_DONE                  /* Tokenization done */
 } Tokenizing_step;
 
@@ -259,6 +286,9 @@ struct Gword_struct
 	Gword **prev;        /* Left-going tree */
 	Gword *chain_next;   /* Next word in the chain of all words */
 
+	/* Disjuncts and connectors point back to their originating Gword(s). */
+	gword_set gword_set_head;
+
 	/* For debug and inspiration. */
 	const char *label;   /* Debug label - code locations of tokenization */
 	size_t node_num;     /* For differentiating words with identical subwords,
