@@ -150,8 +150,8 @@ static inline unsigned int pair_hash(unsigned int table_size,
 	i = cost;
 	i = lw + (i << 6) + (i << 16) - i;
 	i = rw + (i << 6) + (i << 16) - i;
-	i = ((unsigned long) le) + (i << 6) + (i << 16) - i;
-	i = ((unsigned long) re) + (i << 6) + (i << 16) - i;
+	i = ((int)(intptr_t)le) + (i << 6) + (i << 16) - i;
+	i = ((int)(intptr_t)re) + (i << 6) + (i << 16) - i;
 #endif
 
 	return i & (table_size-1);
