@@ -39,7 +39,7 @@
 
 Gword *gword_new(Sentence sent, const char *s)
 {
-	Gword * const gword = malloc(sizeof(*gword));
+	Gword *gword = malloc(sizeof(*gword));
 
 	memset(gword, 0, sizeof(*gword));
 	assert(NULL != gword, "Null-string subword");
@@ -49,11 +49,16 @@ Gword *gword_new(Sentence sent, const char *s)
 	sent->last_word = gword;
 	gword->node_num = sent->gword_node_num++;
 
+	gword->gword_set_head = (gword_set){0};
+	gword->gword_set_head.o_gword = gword;
+
 	return gword;
 }
 
+/* FIXME: Remove it. */
 Gword *empty_word(void)
 {
+	/*
 	static Gword e = {
 		.subword = EMPTY_WORD_MARK,
 		.unsplit_word = &e,
@@ -61,8 +66,8 @@ Gword *empty_word(void)
 		.alternative_id = &e,
 		.status = WS_INDICT,
 	};
-
-	return &e;
+	*/
+	return NULL;
 }
 
 static Gword **gwordlist_resize(Gword **arr, size_t len)
@@ -88,6 +93,26 @@ void gwordlist_append(Gword ***arrp, Gword *p)
 	(*arrp)[n] = p;
 }
 
+/**
+ * Append a Gword list to a given Gword list (w/o duplicates).
+ */
+void gwordlist_append_list(const Gword ***to_word, const Gword **from_word)
+{
+	size_t to_word_arr_len = gwordlist_len(*to_word);
+
+	for (const Gword **f = from_word; NULL != *f; f++)
+	{
+		size_t l;
+
+		/* Note: Must use indexing because to_word may get realloc'ed. */
+		for (l = 0; l < to_word_arr_len; l++)
+			if (*f == (*to_word)[l]) break; /* Filter duplicates. */
+
+		if (l == to_word_arr_len)
+			gwordlist_append((Gword ***)to_word, (Gword *)*f);
+	}
+}
+
 #if 0
 /**
  * Replace "count" words from the position "start" by word "wnew".
@@ -110,12 +135,15 @@ size_t wordgraph_pathpos_len(Wordgraph_pathpos *wp)
 	return len;
 }
 
+/**
+ * `len` is the new length, not counting the terminating null entry.
+ */
 /* FIXME (efficiency): Initially allocate more than 2 elements */
 Wordgraph_pathpos *wordgraph_pathpos_resize(Wordgraph_pathpos *wp,
                                             size_t len)
 {
-	wp = realloc(wp, (len+2) * sizeof(*wp));
-	wp[len+1].word = NULL;
+	wp = realloc(wp, (len+1) * sizeof(*wp));
+	wp[len].word = NULL;
 	return wp;
 }
 
@@ -172,11 +200,15 @@ bool wordgraph_pathpos_add(Wordgraph_pathpos **wp, Gword *p, bool used,
 		}
 	}
 
-	*wp = wordgraph_pathpos_resize(*wp, n);
+	*wp = wordgraph_pathpos_resize(*wp, n+1);
+
 	if (insert_here < n)
 	{
+		/* n+1 because n is the length of the array, not including the
+		 * terminating null entry. We need to protect the terminating null.
+		 */
 		memmove(&(*wp)[insert_here+1], &(*wp)[insert_here],
-		        (n - insert_here) * sizeof (*wpt));
+		        (n+1 - insert_here) * sizeof (*wpt));
 	}
 
 	(*wp)[insert_here].word = p;
@@ -210,6 +242,26 @@ GNUC_UNUSED void print_hier_position(const Gword *word)
 	}
 	err_msg(lg_Debug, "]\n");
 }
+
+/* Debug printout of a wordgraph Gword list. */
+GNUC_UNUSED void gword_set_print(const gword_set *gs)
+{
+	printf("Gword list: ");
+
+	if (NULL == gs)
+	{
+		printf("(null)\n");
+		return;
+	}
+
+	for (; NULL != gs; gs = gs->next)
+	{
+		printf("word %p '%s' unsplit '%s'%s", gs->o_gword, (gs->o_gword)->subword,
+		       (gs->o_gword)->unsplit_word->subword, NULL==gs->next ? "" : ", ");
+	}
+	printf("\n");
+
+}
 #endif
 
 /**
