@@ -29,6 +29,7 @@ static struct
 	int memory;
 	int linkage_limit;
 	int islands_ok;
+	int repeatable_rand;
 	int spell_guess;
 	int short_length;
 	int batch_mode;
@@ -89,10 +90,11 @@ static Switch default_switches[] =
 	{"panic",      Bool, "Use of \"panic mode\"",           &local.panic_mode},
 	{"postscript", Bool, "Generate postscript output",      &local.display_postscript},
 	{"ps-header",  Bool, "Generate postscript header",      &local.display_ps_header},
+	{"rand",       Bool, "Use repeatable random numbers",   &local.repeatable_rand},
 	{"senses",     Bool, "Display of word senses",          &local.display_senses},
 	{"short",      Int,  "Max length of short links",       &local.short_length},
 #if defined HAVE_HUNSPELL || defined HAVE_ASPELL
-	{"spell",      Int, "Use up to this many spell-guesses per unknown word", &local.spell_guess},
+	{"spell",      Int, "Up to this many spell-guesses per unknown word", &local.spell_guess},
 #endif /* HAVE_HUNSPELL */
 	{"timeout",    Int,  "Abort parsing after this many seconds", &local.timeout},
 #ifdef USE_SAT_SOLVER
@@ -183,18 +185,15 @@ static void setival(Switch s, int val)
 	*((int *) s.ptr) = val;
 }
 
-static int x_issue_special_command(const char * line, Command_Options *copts, Dictionary dict)
+static int x_issue_special_command(char * line, Command_Options *copts, Dictionary dict)
 {
-	char *s, myline[1000], *x, *y;
+	char *s, *x, *y;
 	int i, count, j, k;
 	Switch * as = default_switches;
 	Parse_Options opts = copts->popts;
 
-	strncpy(myline, line, sizeof(myline));
-	myline[sizeof(myline)-1] = '\0';
-	clean_up_string(myline);
-
-	s = myline;
+	clean_up_string(line);
+	s = line;
 	j = k = -1;
 	count = 0;
 
@@ -233,68 +232,63 @@ static int x_issue_special_command(const char * line, Command_Options *copts, Di
 			printf("%s turned %s.\n", as[j].description, (ival(as[j]))? "on" : "off");
 			return 0;
 		}
-		else
-		{
-			/* Found an abbreviated command, but it wasn't a boolean */
-			/* Replace the abbreviated command by the full one */
-			/* Basically, this just fixes !v and !h for use below. */
-			strcpy(s, user_command[k].s);
-		}
-	}
 
-	if (strcmp(s, "variables") == 0)
-	{
-		printf(" Variable     Controls                                          Value\n");
-		printf(" --------     --------                                          -----\n");
-		for (i = 0; as[i].string != NULL; i++)
+		/* Found an abbreviated command, but it wasn't a boolean.
+		 * It means it is a user command, to be handled below. */
+
+		if (strcmp(user_command[k].s, "variables") == 0)
 		{
-			printf(" ");
-			left_print_string(stdout, as[i].string, "             ");
-			left_print_string(stdout, as[i].description,
-			            "                                                  ");
-			if (Float == as[i].param_type)
+			printf(" Variable     Controls                                          Value\n");
+			printf(" --------     --------                                          -----\n");
+			for (i = 0; as[i].string != NULL; i++)
 			{
-				/* Float point print! */
-				printf("%5.2f", *((double *)as[i].ptr));
-			}
-			else
-			if ((Bool == as[i].param_type) || Int == as[i].param_type)
-			{
-				printf("%5d", ival(as[i]));
-			}
-			else
-			if (String == as[i].param_type)
-			{
-				printf("%s", *(char **)as[i].ptr);
-			}
-			if (Bool == as[i].param_type)
-			{
-				if (ival(as[i])) printf(" (On)"); else printf(" (Off)");
+				printf(" ");
+				left_print_string(stdout, as[i].string, 13);
+				left_print_string(stdout, as[i].description, 46);
+				if (Float == as[i].param_type)
+				{
+					/* Float point print! */
+					printf("%5.2f", *((double *)as[i].ptr));
+				}
+				else
+				if ((Bool == as[i].param_type) || Int == as[i].param_type)
+				{
+					printf("%5d", ival(as[i]));
+				}
+				else
+				if (String == as[i].param_type)
+				{
+					printf("%s", *(char **)as[i].ptr);
+				}
+				if (Bool == as[i].param_type)
+				{
+					if (ival(as[i])) printf(" (On)"); else printf(" (Off)");
+				}
+				printf("\n");
 			}
 			printf("\n");
+			printf("Toggle a boolean variable as in \"!batch\"; ");
+			printf("set a variable as in \"!width=100\".\n");
+			return 0;
 		}
-		printf("\n");
-		printf("Toggle a boolean variable as in \"!batch\"; ");
-		printf("set a variable as in \"!width=100\".\n");
-		return 0;
-	}
 
-	if (strcmp(s, "help") == 0)
-	{
-		printf("Special commands always begin with \"!\".  Command and variable names\n");
-		printf("can be abbreviated.  Here is a list of the commands:\n\n");
-		for (i=0; user_command[i].s != NULL; i++) {
-			printf(" !");
-			left_print_string(stdout, user_command[i].s, "               ");
-			left_print_string(stdout, user_command[i].str, "                                                    ");
+		if (strcmp(user_command[k].s, "help") == 0)
+		{
+			printf("Special commands always begin with \"!\".  Command and variable names\n");
+			printf("can be abbreviated.  Here is a list of the commands:\n\n");
+			for (i=0; user_command[i].s != NULL; i++) {
+				printf(" !");
+				left_print_string(stdout, user_command[i].s, 15);
+				left_print_string(stdout, user_command[i].str, 52);
+				printf("\n");
+			}
+			printf(" !!<string>      Print all the dictionary words that matches <string>.\n");
+			printf("                 A wildcard * may be used to find multiple matches.\n");
 			printf("\n");
+			printf(" !<var>          Toggle the specified boolean variable.\n");
+			printf(" !<var>=<val>    Assign that value to that variable.\n");
+			return 0;
 		}
-		printf(" !!<string>      Print all the dictionary words that matches <string>.\n");
-		printf("                 A wildcard * may be used to find multiple matches.\n");
-		printf("\n");
-		printf(" !<var>          Toggle the specified boolean variable.\n");
-		printf(" !<var>=<val>    Assign that value to that variable.\n");
-		return 0;
 	}
 
 	if (s[0] == '!')
@@ -387,7 +381,8 @@ static int x_issue_special_command(const char * line, Command_Options *copts, Di
 		}
 		else
 		{
-			printf("Internal error: Unknown variable type %d\n", as[j].param_type);
+			prt_error("Error: Internal error: Unknown variable type %d\n",
+			          as[j].param_type);
 			return -1;
 		}
 	}
@@ -411,7 +406,7 @@ static int x_issue_special_command(const char * line, Command_Options *copts, Di
 		return -1;
 	}
 
-	printf("I can't interpret \"%s\" as a command.  Try \"!help\".\n", myline);
+	printf("I can't interpret \"%s\" as a command.  Try \"!help\".\n", line);
 	return -1;
 }
 
@@ -425,6 +420,7 @@ static void put_opts_in_local_vars(Command_Options* copts)
 	local.memory = parse_options_get_max_memory(opts);;
 	local.linkage_limit = parse_options_get_linkage_limit(opts);
 	local.islands_ok = parse_options_get_islands_ok(opts);
+	local.repeatable_rand = parse_options_get_repeatable_rand(opts);
 	local.spell_guess = parse_options_get_spell_guess(opts);
 	local.short_length = parse_options_get_short_length(opts);
 	local.cost_model = parse_options_get_cost_model_type(opts);
@@ -462,6 +458,7 @@ static void put_local_vars_in_opts(Command_Options* copts)
 	parse_options_set_max_memory(opts, local.memory);
 	parse_options_set_linkage_limit(opts, local.linkage_limit);
 	parse_options_set_islands_ok(opts, local.islands_ok);
+	parse_options_set_repeatable_rand(opts, local.repeatable_rand);
 	parse_options_set_spell_guess(opts, local.spell_guess);
 	parse_options_set_short_length(opts, local.short_length);
 	parse_options_set_cost_model_type(opts, local.cost_model);
@@ -503,8 +500,10 @@ int issue_special_command(const char * line, Command_Options* opts, Dictionary d
 	}
 
 	put_opts_in_local_vars(opts);
-	rc = x_issue_special_command(line, opts, dict);
+	char *cline = strdup(line);
+	rc = x_issue_special_command(cline, opts, dict);
 	put_local_vars_in_opts(opts);
+	free(cline);
 
 	if (save) opts->popts = save;
 	return rc;
